const path = require('path');
const fs = require('fs');
const handlerbars = require('handlebars');
const config = require('./config');

let viewPath = path.join(process.cwd(), 'src', 'views');
let LAYOUT = path.join(viewPath, 'layout.html');
const devViewPath = path.join(__dirname, 'views');
const devLayout = path.join(devViewPath, 'layout.html');

const checkExists = (filePath) => {
  fs.accessSync(filePath, fs.F_OK);
};

try {
  checkExists(viewPath);
} catch (e) {
  viewPath = path.join(__dirname, 'views');
  LAYOUT = path.join(viewPath, 'layout.html');
}

const read = (file) => {
  return new Promise((resolve, reject) => {
    fs.readFile(file, (err, data) => {
      if (err) {
        reject(err);
      }
      resolve(data.toString());
    });
  });
};

const readTemplates = (viewName, res) => {
  let view;
  let layout;
  if (process.env.NODE_ENV === 'development' && res.url.includes(config.dev.routes)) {
    view = path.join(devViewPath, `${viewName}.html`);
    layout = devLayout;
  } else {
    view = path.join(viewPath, `${viewName}.html`);
    layout = LAYOUT;
  }

  return Promise.all([read(layout), read(view)]);
};

module.exports = function (viewName, viewData) {
  return readTemplates(viewName, this)
    .then((data) => {
      const [l, v] = data;
      const viewTemp = handlerbars.compile(v);
      const vContext = { data: viewData.content };
      const vhtml = viewTemp(vContext);
      const layout = handlerbars.compile(l);
      const lContext = { title: viewData.title, body: vhtml };
      const lhtml = layout(lContext);
      this.writeHead(200, { 'Content-Type': 'text/html' });
      this.end(lhtml);
    })
    .catch((err) => {
      this.writeHead(500, { 'Content-Type': 'text/html' });
      this.end(`500 error: ${err}`);
    });
};
