const path = require('path');
const requireIndex = require('requireindex');
const fs = require('fs');
const config = require('../config');

let routes;
const devRoutes = requireIndex(path.join(__dirname));

try {
  routes = requireIndex(path.join(process.cwd(), 'src', 'routes'));
} catch (e) {
  routes = requireIndex(path.join(__dirname));
}

const checkExists = (filePath) => {
  try {
    fs.accessSync(filePath, fs.F_OK);
    return fs.readFileSync(filePath);
  } catch (e) {
    return null;
  }
};

module.exports = (req, res) => {
  let rt = routes[`${req.url.split('/').pop()}`];
  let staticRes = checkExists(path.join(process.cwd(), 'public', req.url));
  if (req.url === '/') {
    rt = routes.home;
  }

  // Setup dev routes
  if (process.env.NODE_ENV === 'development' && req.url.includes(config.dev.routes)) {
    const route = req.url.split('/').pop();
    const staticRoute = req.url.split('/').slice(2, req.url.split('/').length).join('/');
    staticRes = checkExists(path.join(__dirname, '..', '..', 'public', staticRoute));
    if (staticRes) {
      return res.end(staticRes);
    }

    if (req.url === '/jolly' || req.url === '/jolly/') {
      return devRoutes.home(req, res);
    }

    if (typeof devRoutes[route] === 'function') {
      return devRoutes[route](req, res);
    }

    return devRoutes.notfound(req, res);
  }

  if (typeof rt === 'function') {
    rt(req, res);
  } else if (staticRes) {
    res.end(staticRes);
  } else {
    routes.notfound(req, res);
  }
};
